package com.abi.task.common.tablestore.common;

import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.IStringUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;

import java.lang.reflect.Field;

/**
 * 查询table store的入参
 * @author AndyTan
 */
@Data
@Slf4j
public class TableStoreReq {

    /**
     * 表名
     */
    String tableName;

    /**
     * 索引名
     */
    String indexName;

    /**
     * 匹配的字段名
     */
    String fieldName;

    /**
     * 匹配的字段值
     */
    Object fieldValue;

    /**
     * 大于xxx
     */
    Object greaterThan;

    /**
     * 小于xxx
     */
    Object lessThan;

    /**
     * 每页条数
     */
    Integer pageCount = 10;

    /**
     * 第几页
     */
    Integer pageNo = 1;

    /**
     * 需要返回总数量
     */
    Boolean getTotalCount = false;

    /**
     * 是否逆序
     */
    Boolean sortByFieldDesc = false;

    /**
     * 印射对象类
     */
    Class clz;

    /**
     * 无参构造函数
     */
    public TableStoreReq(){}

    /**
     * 构造函数
     */
    public <T extends TableStoreEntity> TableStoreReq(Class<T> clz, String fieldName){
        Field field = null;
        try{
            field = clz.getDeclaredField(fieldName);
        }catch (Exception e){
            log.info("找不到字段",e);
        }

        //表名
        TableStore tableStore = clz.getAnnotation(TableStore.class);
        String tableName = tableStore.tableName();
        //索引
        TableStoreIndex tableStoreIndex = field.getAnnotation(TableStoreIndex.class);
        String indexName = tableStoreIndex.indexName();
        //字段名转下划线
        fieldName = IStringUtil.camelToUnderline(fieldName);

        this.tableName = tableName;
        this.indexName = indexName;
        this.fieldName = fieldName;
        this.clz = clz;
    }

    /**
     * 构造函数
     * @param tableName 表名
     * @param indexName 索引名
     * @param fieldName 匹配的字段名
     */
    public TableStoreReq(String tableName,String indexName,String fieldName){
        this.tableName = tableName;
        this.indexName = indexName;
        this.fieldName = fieldName;
    }

    /**
     * 设置范围
     * @param greaterThan
     * @param lessThan
     * @return
     */
    public TableStoreReq range(Object greaterThan,Object lessThan){
        this.setGreaterThan(greaterThan);
        this.setLessThan(lessThan);
        return this;
    }

    /**
     * 设置等于
     * @param fieldValue
     * @return
     */
    public TableStoreReq equal(Object fieldValue){
        this.setFieldValue(fieldValue);
        return this;
    }

    public TableStoreReq page(Integer pageNo,Integer pageCount){
        this.pageNo = pageNo;
        this.pageCount = pageCount;
        return this;
    }

    /**
     * 是否逆序
     * @return
     */
    public TableStoreReq isDesc(){
        this.setSortByFieldDesc(true);
        return this;
    }

    /**
     * 是否需要总数
     * @return
     */
    public TableStoreReq isNeedTotalCount(){
        this.setGetTotalCount(true);
        return this;
    }


}
